package com.sun.media.sound;

import java.io.SequenceInputStream;
import java.io.ByteArrayInputStream;
import java.io.DataOutputStream;
import java.io.ByteArrayOutputStream;
import javax.sound.sampled.AudioSystem;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioFileFormat;

public final class WaveFileWriter extends SunFileWriter
{
    static final int RIFF_MAGIC = 1380533830;
    static final int WAVE_MAGIC = 1463899717;
    static final int FMT_MAGIC = 1718449184;
    static final int DATA_MAGIC = 1684108385;
    static final int WAVE_FORMAT_UNKNOWN = 0;
    static final int WAVE_FORMAT_PCM = 1;
    static final int WAVE_FORMAT_ADPCM = 2;
    static final int WAVE_FORMAT_ALAW = 6;
    static final int WAVE_FORMAT_MULAW = 7;
    static final int WAVE_FORMAT_OKI_ADPCM = 16;
    static final int WAVE_FORMAT_DIGISTD = 21;
    static final int WAVE_FORMAT_DIGIFIX = 22;
    static final int WAVE_IBM_FORMAT_MULAW = 257;
    static final int WAVE_IBM_FORMAT_ALAW = 258;
    static final int WAVE_IBM_FORMAT_ADPCM = 259;
    static final int WAVE_FORMAT_DVI_ADPCM = 17;
    static final int WAVE_FORMAT_SX7383 = 7175;
    
    public WaveFileWriter() {
        super(new AudioFileFormat.Type[] { AudioFileFormat.Type.WAVE });
    }
    
    @Override
    public AudioFileFormat.Type[] getAudioFileTypes(AudioInputStream audioInputStream) {
        AudioFileFormat.Type[] array = new AudioFileFormat.Type[this.types.length];
        System.arraycopy(this.types, 0, array, 0, this.types.length);
        AudioFormat.Encoding encoding = audioInputStream.getFormat().getEncoding();
        if (AudioFormat.Encoding.ALAW.equals(encoding) || AudioFormat.Encoding.ULAW.equals(encoding) || AudioFormat.Encoding.PCM_SIGNED.equals(encoding) || AudioFormat.Encoding.PCM_UNSIGNED.equals(encoding)) {
            return array;
        }
        return new AudioFileFormat.Type[0];
    }
    
    @Override
    public int write(AudioInputStream audioInputStream, AudioFileFormat.Type type, OutputStream outputStream) throws IOException {
        WaveFileFormat waveFileFormat = (WaveFileFormat)this.getAudioFileFormat(type, audioInputStream);
        if (audioInputStream.getFrameLength() == -1L) {
            throw new IOException("stream length not specified");
        }
        return this.writeWaveFile(audioInputStream, waveFileFormat, outputStream);
    }
    
    @Override
    public int write(AudioInputStream audioInputStream, AudioFileFormat.Type type, File file) throws IOException {
        WaveFileFormat waveFileFormat = (WaveFileFormat)this.getAudioFileFormat(type, audioInputStream);
        BufferedOutputStream bufferedOutputStream = new BufferedOutputStream(new FileOutputStream(file), 4096);
        int writeWaveFile = this.writeWaveFile(audioInputStream, waveFileFormat, bufferedOutputStream);
        bufferedOutputStream.close();
        if (waveFileFormat.getByteLength() == -1) {
            int n = writeWaveFile - waveFileFormat.getHeaderSize();
            int n2 = n + waveFileFormat.getHeaderSize() - 8;
            RandomAccessFile randomAccessFile = new RandomAccessFile(file, "rw");
            randomAccessFile.skipBytes(4);
            randomAccessFile.writeInt(this.big2little(n2));
            randomAccessFile.skipBytes(12 + WaveFileFormat.getFmtChunkSize(waveFileFormat.getWaveType()) + 4);
            randomAccessFile.writeInt(this.big2little(n));
            randomAccessFile.close();
        }
        return writeWaveFile;
    }
    
    private AudioFileFormat getAudioFileFormat(AudioFileFormat.Type type, AudioInputStream audioInputStream) {
        AudioFormat.Encoding pcm_SIGNED = AudioFormat.Encoding.PCM_SIGNED;
        AudioFormat format = audioInputStream.getFormat();
        AudioFormat.Encoding encoding = format.getEncoding();
        if (!this.types[0].equals(type)) {
            throw new IllegalArgumentException("File type " + type + " not supported.");
        }
        int n = 1;
        AudioFormat.Encoding encoding2;
        int sampleSizeInBits;
        if (AudioFormat.Encoding.ALAW.equals(encoding) || AudioFormat.Encoding.ULAW.equals(encoding)) {
            encoding2 = encoding;
            sampleSizeInBits = format.getSampleSizeInBits();
            if (encoding.equals(AudioFormat.Encoding.ALAW)) {
                n = 6;
            }
            else {
                n = 7;
            }
        }
        else if (format.getSampleSizeInBits() == 8) {
            encoding2 = AudioFormat.Encoding.PCM_UNSIGNED;
            sampleSizeInBits = 8;
        }
        else {
            encoding2 = AudioFormat.Encoding.PCM_SIGNED;
            sampleSizeInBits = format.getSampleSizeInBits();
        }
        AudioFormat o = new AudioFormat(encoding2, format.getSampleRate(), sampleSizeInBits, format.getChannels(), format.getFrameSize(), format.getFrameRate(), false);
        int n2;
        if (audioInputStream.getFrameLength() != -1L) {
            n2 = (int)audioInputStream.getFrameLength() * format.getFrameSize() + WaveFileFormat.getHeaderSize(n);
        }
        else {
            n2 = -1;
        }
        return new WaveFileFormat(AudioFileFormat.Type.WAVE, n2, o, (int)audioInputStream.getFrameLength());
    }
    
    private int writeWaveFile(InputStream inputStream, WaveFileFormat waveFileFormat, OutputStream outputStream) throws IOException {
        int n = 0;
        InputStream fileStream = this.getFileStream(waveFileFormat, inputStream);
        byte[] array = new byte[4096];
        int byteLength = waveFileFormat.getByteLength();
        int read;
        while ((read = fileStream.read(array)) >= 0) {
            if (byteLength > 0) {
                if (read < byteLength) {
                    outputStream.write(array, 0, read);
                    n += read;
                    byteLength -= read;
                }
                else {
                    outputStream.write(array, 0, byteLength);
                    n += byteLength;
                    break;
                }
            }
            else {
                outputStream.write(array, 0, read);
                n += read;
            }
        }
        return n;
    }
    
    private InputStream getFileStream(WaveFileFormat waveFileFormat, InputStream inputStream) throws IOException {
        AudioFormat format = waveFileFormat.getFormat();
        int headerSize = waveFileFormat.getHeaderSize();
        int v = 1380533830;
        int v2 = 1463899717;
        int v3 = 1718449184;
        int fmtChunkSize = WaveFileFormat.getFmtChunkSize(waveFileFormat.getWaveType());
        int n = (short)waveFileFormat.getWaveType();
        int n2 = (short)format.getChannels();
        int n3 = (short)format.getSampleSizeInBits();
        int n4 = (int)format.getSampleRate();
        int frameSize = format.getFrameSize();
        int n5 = (int)format.getFrameRate();
        int n6 = n2 * n3 * n4 / 8;
        int n7 = (short)(n3 / 8 * n2);
        int v4 = 1684108385;
        int n8 = waveFileFormat.getFrameLength() * frameSize;
        waveFileFormat.getByteLength();
        int n9 = n8 + headerSize - 8;
        if ((inputStream instanceof AudioInputStream)) {
            AudioFormat format2 = ((AudioInputStream)inputStream).getFormat();
            AudioFormat.Encoding encoding = format2.getEncoding();
            if (AudioFormat.Encoding.PCM_SIGNED.equals(encoding) && n3 == 8) {
                n = 1;
                inputStream = AudioSystem.getAudioInputStream(new AudioFormat(AudioFormat.Encoding.PCM_UNSIGNED, format2.getSampleRate(), format2.getSampleSizeInBits(), format2.getChannels(), format2.getFrameSize(), format2.getFrameRate(), false), (AudioInputStream)inputStream);
            }
            if (((AudioFormat.Encoding.PCM_SIGNED.equals(encoding) && format2.isBigEndian()) || (AudioFormat.Encoding.PCM_UNSIGNED.equals(encoding) && !format2.isBigEndian()) || (AudioFormat.Encoding.PCM_UNSIGNED.equals(encoding) && format2.isBigEndian())) && n3 != 8) {
                n = 1;
                inputStream = AudioSystem.getAudioInputStream(new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, format2.getSampleRate(), format2.getSampleSizeInBits(), format2.getChannels(), format2.getFrameSize(), format2.getFrameRate(), false), (AudioInputStream)inputStream);
            }
        }
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        DataOutputStream o = new DataOutputStream(out);
        o.writeInt(v);
        o.writeInt(this.big2little(n9));
        o.writeInt(v2);
        o.writeInt(v3);
        o.writeInt(this.big2little(fmtChunkSize));
        o.writeShort((int)this.big2littleShort((short)n));
        o.writeShort((int)this.big2littleShort((short)n2));
        o.writeInt(this.big2little(n4));
        o.writeInt(this.big2little(n6));
        o.writeShort((int)this.big2littleShort((short)n7));
        o.writeShort((int)this.big2littleShort((short)n3));
        if (n != 1) {
            o.writeShort(0);
        }
        o.writeInt(v4);
        o.writeInt(this.big2little(n8));
        o.close();
        return new SequenceInputStream(new ByteArrayInputStream(out.toByteArray()), new SunFileWriter.NoCloseInputStream(inputStream));
    }
}
